@@ -2,7 +2,7 @@
  *
  * raptor.h - Redland Parser Toolkit for RDF (Raptor) interfaces and definition
  *
- * Copyright (C) 2000-2006, David Beckett http://purl.org/net/dajobe/
+ * Copyright (C) 2000-2007, David Beckett http://purl.org/net/dajobe/
  * Copyright (C) 2000-2005, University of Bristol, UK http://www.bristol.ac.uk/
  * 
  * This package is Free Software and part of Redland http://librdf.org/
@@ -45,6 +45,10 @@ extern "C" {
  *
  */
 #ifdef WIN32
+#  ifdef __GNUC__
+#    undef _declspec
+#    define _declspec(x) __declspec(x)
+#  endif
 #  ifdef RAPTOR_STATIC
 #    define RAPTOR_API
 #  else
@@ -54,6 +58,12 @@ extern "C" {
 #      define RAPTOR_API _declspec(dllimport)
 #    endif
 #  endif
+#elif defined(__SYMBIAN32__)
+#  ifdef RAPTOR_INTERNAL
+#    define RAPTOR_API __declspec(dllexport)
+#  else
+#    define RAPTOR_API __declspec(dllimport)
+#  endif
 #else
 #  define RAPTOR_API
 #endif
@@ -72,6 +82,20 @@ extern "C" {
 #define RAPTOR_DEPRECATED
 #endif
 
+/**
+ * RAPTOR_PRINTF_FORMAT:
+ * @string_index: ignore me
+ * @first_to_check_index: ignore me
+ *
+ * Internal macro
+ */
+#if __GNUC__ > 2 || (__GNUC__ == 2 && __GNUC_MINOR__ > 4)
+#define RAPTOR_PRINTF_FORMAT(string_index, first_to_check_index) \
+  __attribute__((__format__(__printf__, string_index, first_to_check_index)))
+#else
+#define RAPTOR_PRINTF_FORMAT(string_index, first_to_check_index)
+#endif
+
 /**
  * raptor_uri:
  *
@@ -95,6 +119,10 @@ RAPTOR_API
 extern const unsigned int raptor_version_release;
 RAPTOR_API
 extern const unsigned int raptor_version_decimal;
+RAPTOR_API
+extern const char * const raptor_license_string;
+RAPTOR_API
+extern const char * const raptor_home_url_string;
 
 RAPTOR_API
 extern const unsigned char * const raptor_xml_namespace_uri;
@@ -197,8 +225,8 @@ typedef raptor_parser raptor_ntriples_parser;
  * raptor_identifier_type:
  * @RAPTOR_IDENTIFIER_TYPE_RESOURCE:    Resource URI (e.g. <literal>rdf:about</literal>)
  * @RAPTOR_IDENTIFIER_TYPE_ANONYMOUS:   <literal>_:foo</literal> N-Triples, or generated
- * @RAPTOR_IDENTIFIER_TYPE_PREDICATE:   predicate URI.  WARNING: Will not be generated in in Raptor 1.4.9 or newer.  Instead a #RAPTOR_IDENTIFIER_RESOURCE will be returned.
- * @RAPTOR_IDENTIFIER_TYPE_ORDINAL:     <literal>rdf:li</literal>, <literal>rdf:_</literal><emphasis>n</emphasis>.  No longer generated in any parser in Raptor 1.4.10+, instead a #RAPTOR_IDENTIFIER_RESOURCE is returned.
+ * @RAPTOR_IDENTIFIER_TYPE_PREDICATE:   predicate URI.  WARNING: Will not be generated in in Raptor 1.4.9 or newer.  Instead a #RAPTOR_IDENTIFIER_TYPE_RESOURCE will be returned.
+ * @RAPTOR_IDENTIFIER_TYPE_ORDINAL:     <literal>rdf:li</literal>, <literal>rdf:_</literal><emphasis>n</emphasis>.  No longer generated in any parser in Raptor 1.4.10+, instead a #RAPTOR_IDENTIFIER_TYPE_RESOURCE is returned.
  * @RAPTOR_IDENTIFIER_TYPE_LITERAL:     regular literal
  * @RAPTOR_IDENTIFIER_TYPE_XML_LITERAL: <literal>rdf:parseType="Literal"</literal>.  No longer generated by any parser in Raptor 1.4.8+, instead a #RAPTOR_IDENTIFIER_TYPE_LITERAL is returned with a datatype of <literal>rdf:XMLLiteral</literal>.
  * @RAPTOR_IDENTIFIER_TYPE_UNKNOWN:     Internal
@@ -317,7 +345,26 @@ typedef struct {
  *   for each indent level when serializing with auto indent.
  * @RAPTOR_FEATURE_WRITER_XML_VERSION: Integer XML version XML 1.0 (10) or XML 1.1 (11)
  * @RAPTOR_FEATURE_WRITER_XML_DECLARATION: Write XML 1.0 or 1.1 declaration.
- * @RAPTOR_FEATURE_NO_NET: Deny network requests
+ * @RAPTOR_FEATURE_NO_NET: Deny network requests.
+ * @RAPTOR_FEATURE_RESOURCE_BORDER: Border color of resource
+ *   nodes for GraphViz DOT serializer.
+ * @RAPTOR_FEATURE_LITERAL_BORDER: Border color of literal nodes
+ *   for GraphViz DOT serializer.
+ * @RAPTOR_FEATURE_BNODE_BORDER: Border color of blank nodes for
+ *   GraphViz DOT serializer.
+ * @RAPTOR_FEATURE_RESOURCE_FILL: Fill color of resource nodes
+ *   for GraphViz DOT serializer.
+ * @RAPTOR_FEATURE_LITERAL_FILL: Fill color of literal nodes for
+ *   GraphViz DOT serializer.
+ * @RAPTOR_FEATURE_BNODE_FILL: Fill color of blank nodes for
+ *   GraphViz DOT serializer.
+ * @RAPTOR_FEATURE_HTML_TAG_SOUP: Use a lax HTML parser if an XML parser
+ *   fails when read HTML for GRDDL parser.
+ * @RAPTOR_FEATURE_MICROFORMATS: Look for microformats for GRDDL parser.
+ * @RAPTOR_FEATURE_HTML_LINK: Look for head &lt;link&gt; to type rdf/xml
+ *   for GRDDL parser.
+ * @RAPTOR_FEATURE_WWW_TIMEOUT: Set timeout for internal WWW URI requests
+ *   for GRDDL parser.
  * @RAPTOR_FEATURE_LAST: Internal
  *
  * Raptor parser, serializer or XML writer features.
@@ -341,7 +388,17 @@ typedef enum {
   RAPTOR_FEATURE_WRITER_XML_VERSION,
   RAPTOR_FEATURE_WRITER_XML_DECLARATION,
   RAPTOR_FEATURE_NO_NET,
-  RAPTOR_FEATURE_LAST=RAPTOR_FEATURE_NO_NET
+  RAPTOR_FEATURE_RESOURCE_BORDER,
+  RAPTOR_FEATURE_LITERAL_BORDER,
+  RAPTOR_FEATURE_BNODE_BORDER,
+  RAPTOR_FEATURE_RESOURCE_FILL,
+  RAPTOR_FEATURE_LITERAL_FILL,
+  RAPTOR_FEATURE_BNODE_FILL,
+  RAPTOR_FEATURE_HTML_TAG_SOUP,
+  RAPTOR_FEATURE_MICROFORMATS,
+  RAPTOR_FEATURE_HTML_LINK,
+  RAPTOR_FEATURE_WWW_TIMEOUT,
+  RAPTOR_FEATURE_LAST=RAPTOR_FEATURE_WWW_TIMEOUT
 } raptor_feature;
 
 
@@ -485,6 +542,18 @@ typedef void (*raptor_free_uri_func) (void *context, raptor_uri *uri);
  */
 typedef int (*raptor_uri_equals_func) (void *context, raptor_uri* uri1, raptor_uri* uri2);
 
+/**
+ * raptor_uri_compare_func:
+ * @context: URI context data
+ * @uri1: URI object 1
+ * @uri2: URI object 2
+ *
+ * Handler function for implementing raptor_uri_equals().
+ *
+ * Return value: -1 if uri1 < uri2, 0 if equal, 1 if uri1 > uri2
+ */
+typedef int (*raptor_uri_compare_func) (void *context, raptor_uri* uri1, raptor_uri* uri2);
+
 /**
  * raptor_uri_copy_func:
  * @context: URI context data
@@ -528,27 +597,30 @@ typedef unsigned char* (*raptor_uri_as_counted_string_func)(void *context, rapto
  * @new_uri_for_rdf_concept: function for raptor_new_uri_for_rdf_concept()
  * @free_uri: function for raptor_free_uri()
  * @uri_equals: function for raptor_uri_equals()
+ * @uri_compare: function for raptor_uri_compare()
  * @uri_copy: function for raptor_uri_copy()
  * @uri_as_string: function for raptor_uri_as_string()
  * @uri_as_counted_string: function for raptor_uri_as_counted_string()
- * @initialised: Internal
+ * @initialised: API version - set to API version implemented: 1..2
  *
  * URI implementation handler structure.
  */
 typedef struct {
-  /* constructors */
+  /* constructors - URI Interface V1 */
   raptor_new_uri_func                     new_uri;
   raptor_new_uri_from_uri_local_name_func new_uri_from_uri_local_name;
   raptor_new_uri_relative_to_base_func    new_uri_relative_to_base;
   raptor_new_uri_for_rdf_concept_func     new_uri_for_rdf_concept;
-  /* destructor */
+  /* destructor - URI Interface V1 */
   raptor_free_uri_func                    free_uri;
-  /* methods */
+  /* methods - URI Interface V1 */
   raptor_uri_equals_func                  uri_equals;
   raptor_uri_copy_func                    uri_copy; /* well, copy constructor */
   raptor_uri_as_string_func               uri_as_string;
   raptor_uri_as_counted_string_func       uri_as_counted_string;
   int initialised;
+  /* methods - URI Interface V2 */
+  raptor_uri_compare_func                 uri_compare;
 } raptor_uri_handler;
 
 
@@ -597,6 +669,17 @@ typedef void (*raptor_message_handler)(void *user_data, raptor_locator* locator,
  */
 typedef void (*raptor_statement_handler)(void *user_data, const raptor_statement *statement);
 
+/**
+ * raptor_graph_handler:
+ * @user_data: user data
+ * @graph: graph to report, 0 for the default graph
+ *
+ * Named graph reporting handler function. Due to historic reasons the named graph
+ * API is separated from the statement handler. A graph is reported after all its 
+ * statements.
+ */
+typedef void (*raptor_graph_handler)(void *user_data, raptor_uri *graph);
+
 /**
  * raptor_generate_id_handler:
  * @user_data: user data
@@ -645,6 +728,18 @@ typedef void (*raptor_www_write_bytes_handler)(raptor_www* www, void *userdata,
  */
 typedef void (*raptor_www_content_type_handler)(raptor_www* www, void *userdata, const char *content_type);
 
+/**
+ * raptor_www_final_uri_handler:
+ * @www: WWW object
+ * @userdata: user data
+ * @final_uri: final URI seen
+ *
+ * Receiving the final resolved URI from a WWW retrieval
+ *
+ * Set by raptor_www_set_final_uri_handler().
+ */
+typedef void (*raptor_www_final_uri_handler)(raptor_www* www, void *userdata, raptor_uri *final_uri);
+
 /**
  * raptor_uri_filter_func:
  * @user_data: user data
@@ -697,6 +792,8 @@ void raptor_set_warning_handler(raptor_parser* parser, void *user_data, raptor_m
 RAPTOR_API
 void raptor_set_statement_handler(raptor_parser* parser, void *user_data, raptor_statement_handler handler);
 RAPTOR_API
+void raptor_set_graph_handler(raptor_parser* parser, void *user_data, raptor_graph_handler handler);
+RAPTOR_API
 void raptor_set_generate_id_handler(raptor_parser* parser, void *user_data, raptor_generate_id_handler handler);
 RAPTOR_API
 void raptor_set_namespace_handler(raptor_parser* parser, void *user_data, raptor_namespace_handler handler);
@@ -777,6 +874,8 @@ void raptor_set_parser_strict(raptor_parser* rdf_parser, int is_strict);
 RAPTOR_API
 const char* raptor_parser_get_accept_header(raptor_parser* rdf_parser);
 
+unsigned char* raptor_parser_generate_id(raptor_parser *rdf_parser, raptor_genid_type type);
+
 /* Get serializer names */
 RAPTOR_API
 int raptor_serializers_enumerate(const unsigned int counter, const char **name, const char **label, const char **mime_type, const unsigned char **uri_string);
@@ -848,6 +947,8 @@ void raptor_free_uri(raptor_uri *uri);
 RAPTOR_API
 int raptor_uri_equals(raptor_uri* uri1, raptor_uri* uri2);
 RAPTOR_API
+int raptor_uri_compare(raptor_uri* uri1, raptor_uri* uri2);
+RAPTOR_API
 raptor_uri* raptor_uri_copy(raptor_uri *uri);
 RAPTOR_API
 unsigned char* raptor_uri_as_string(raptor_uri *uri);
@@ -879,6 +980,8 @@ const char* raptor_ntriples_term_as_string(raptor_ntriples_term_type term);
 RAPTOR_API
 int raptor_iostream_write_string_ntriples(raptor_iostream *iostr, const unsigned char *string, size_t len, const char delim);
 RAPTOR_API
+void raptor_iostream_write_string_turtle(raptor_iostream *iostr, const unsigned char *string, size_t len);
+RAPTOR_API
 void raptor_iostream_write_statement_ntriples(raptor_iostream* iostr, const raptor_statement *statement);
 RAPTOR_API
 int raptor_xml_any_escape_string(const unsigned char *string, size_t len, unsigned char *buffer, size_t length, char quote, int xml_version, raptor_simple_message_handler error_handler, void *error_data);
@@ -890,7 +993,7 @@ RAPTOR_API
 int raptor_iostream_write_xml_escaped_string(raptor_iostream* iostr, const unsigned char *string, size_t len, char quote, raptor_simple_message_handler error_handler, void *error_data);
 
 RAPTOR_API
-char* raptor_vsnprintf(const char *message, va_list arguments);
+char* raptor_vsnprintf(const char *message, va_list arguments) RAPTOR_PRINTF_FORMAT(1, 0);
 
 RAPTOR_API
 int raptor_xml_name_check(const unsigned char *string, size_t length, int xml_version);
@@ -922,7 +1025,7 @@ RAPTOR_API
 unsigned char* raptor_uri_to_string(raptor_uri *uri);
 
 RAPTOR_API
-void raptor_uri_set_handler(const raptor_uri_handler *handler, void *context);
+void raptor_uri_set_handler(raptor_uri_handler *handler, void *context);
 RAPTOR_API
 void raptor_uri_get_handler(raptor_uri_handler **handler, void **context);
 
@@ -989,10 +1092,14 @@ void raptor_www_set_write_bytes_handler(raptor_www *www, raptor_www_write_bytes_
 RAPTOR_API
 void raptor_www_set_content_type_handler(raptor_www *www, raptor_www_content_type_handler handler, void *user_data);
 RAPTOR_API
+void raptor_www_set_final_uri_handler(raptor_www* www, raptor_www_final_uri_handler handler, void *user_data);
+RAPTOR_API
 void raptor_www_set_error_handler(raptor_www *www, raptor_message_handler error_handler, void *error_data);
 RAPTOR_API
 void raptor_www_set_uri_filter(raptor_www* www, raptor_uri_filter_func filter, void* user_data);
 RAPTOR_API
+void raptor_www_set_connection_timeout(raptor_www* www, int timeout);
+RAPTOR_API
 int raptor_www_fetch(raptor_www *www, raptor_uri *uri);
 RAPTOR_API
 int raptor_www_fetch_to_string(raptor_www *www, raptor_uri *uri, void **string_p, size_t *length_p, void *(*malloc_handler)(size_t size));
@@ -1000,6 +1107,8 @@ RAPTOR_API
 void* raptor_www_get_connection(raptor_www *www);
 RAPTOR_API
 void raptor_www_abort(raptor_www *www, const char *reason);
+RAPTOR_API
+raptor_uri* raptor_www_get_final_uri(raptor_www* www);
 
 
 /* raptor_qname - XML qnames */
@@ -1028,7 +1137,7 @@ RAPTOR_API
 raptor_namespace_stack* raptor_new_namespaces(raptor_uri_handler *uri_handler, void *uri_context, raptor_simple_message_handler error_handler, void *error_data, int defaults);
 
 RAPTOR_API
-void raptor_namespaces_init(raptor_namespace_stack *nstack, raptor_uri_handler *uri_handler, void *uri_context, raptor_simple_message_handler error_handler, void *error_data, int defaults);
+int raptor_namespaces_init(raptor_namespace_stack *nstack, raptor_uri_handler *uri_handler, void *uri_context, raptor_simple_message_handler error_handler, void *error_data, int defaults);
 RAPTOR_API
 void raptor_namespaces_clear(raptor_namespace_stack *nstack);
 RAPTOR_API
@@ -1145,19 +1254,27 @@ void raptor_sequence_print(raptor_sequence* seq, FILE* fh);
 RAPTOR_API
 int raptor_sequence_join(raptor_sequence* dest, raptor_sequence *src);
 
+
 /* Unicode and UTF8 */
+
+/**
+ * raptor_unichar:
+ *
+ * raptor Unicode codepoint
+ */
+typedef unsigned long raptor_unichar;
 RAPTOR_API
-int raptor_unicode_char_to_utf8(unsigned long c, unsigned char *output);
+int raptor_unicode_char_to_utf8(raptor_unichar c, unsigned char *output);
 RAPTOR_API
-int raptor_utf8_to_unicode_char(unsigned long *output, const unsigned char *input, int length);
+int raptor_utf8_to_unicode_char(raptor_unichar *output, const unsigned char *input, int length);
 RAPTOR_API
-int raptor_unicode_is_xml11_namestartchar(long c);
+int raptor_unicode_is_xml11_namestartchar(raptor_unichar c);
 RAPTOR_API
-int raptor_unicode_is_xml10_namestartchar(long c);
+int raptor_unicode_is_xml10_namestartchar(raptor_unichar c);
 RAPTOR_API
-int raptor_unicode_is_xml11_namechar(long c);
+int raptor_unicode_is_xml11_namechar(raptor_unichar c);
 RAPTOR_API
-int raptor_unicode_is_xml10_namechar(long c);
+int raptor_unicode_is_xml10_namechar(raptor_unichar c);
 RAPTOR_API
 int raptor_utf8_check(const unsigned char *string, size_t length);
 
@@ -1300,6 +1417,8 @@ int raptor_feature_value_type(const raptor_feature feature);
 RAPTOR_API
 raptor_xml_element* raptor_new_xml_element(raptor_qname* name, const unsigned char* xml_language, raptor_uri* xml_base);
 RAPTOR_API
+raptor_xml_element* raptor_new_xml_element_from_namespace_local_name(raptor_namespace *ns, const unsigned char *name, const unsigned char *xml_language, raptor_uri *xml_base);
+RAPTOR_API
 void raptor_free_xml_element(raptor_xml_element *element);
 RAPTOR_API
 raptor_qname* raptor_xml_element_get_name(raptor_xml_element *xml_element);
