@@ -65,8 +65,10 @@ raptor_free_serializer_factory(raptor_serializer_factory* factory)
   if(factory->finish_factory)
     factory->finish_factory(factory);
   
-  RAPTOR_FREE(raptor_serializer_factory, (void*)factory->name);
-  RAPTOR_FREE(raptor_serializer_factory, (void*)factory->label);
+  if(factory->name)
+    RAPTOR_FREE(raptor_serializer_factory, (void*)factory->name);
+  if(factory->label)
+    RAPTOR_FREE(raptor_serializer_factory, (void*)factory->label);
   if(factory->alias)
     RAPTOR_FREE(raptor_serializer_factory, (void*)factory->alias);
   if(factory->mime_type)
@@ -84,6 +86,10 @@ void
 raptor_serializers_init(void)
 {
   serializers=raptor_new_sequence((raptor_sequence_free_handler *)raptor_free_serializer_factory, NULL);
+  if(!serializers) {
+    raptor_finish();
+    RAPTOR_FATAL1("Out of memory\n");
+  }
 
   /* raptor_init_serializer_simple(); */
 
@@ -123,8 +129,10 @@ raptor_serializers_init(void)
 void
 raptor_serializers_finish(void)
 {
-  raptor_free_sequence(serializers);
-  serializers=NULL;
+  if(serializers) {
+    raptor_free_sequence(serializers);
+    serializers=NULL;
+  }
 }
 
 
@@ -154,8 +162,7 @@ raptor_serializer_register_factory(const char *name, const char *label,
 #if defined(RAPTOR_DEBUG) && RAPTOR_DEBUG > 1
   RAPTOR_DEBUG4("Received registration for syntax serializer %s '%s' with alias '%s'\n", 
                 name, label, (alias ? alias : "none"));
-  RAPTOR_DEBUG4(raptor_serializer_register_factory,
-                "MIME type %s, URI %s\n", 
+  RAPTOR_DEBUG3("MIME type %s, URI %s\n", 
                 (mime_type ? mime_type : "none"),
                 (uri_string ? uri_string : "none"));
 #endif
@@ -173,62 +180,62 @@ raptor_serializer_register_factory(const char *name, const char *label,
   serializer=(raptor_serializer_factory*)RAPTOR_CALLOC(raptor_serializer_factory, 1,
                                                sizeof(raptor_serializer_factory));
   if(!serializer)
-    RAPTOR_FATAL1("Out of memory\n");
+    goto fail_noserializer;
 
   name_copy=(char*)RAPTOR_CALLOC(cstring, strlen(name)+1, 1);
-  if(!name_copy) {
-    RAPTOR_FREE(raptor_serializer, serializer);
-    RAPTOR_FATAL1("Out of memory\n");
-  }
+  if(!name_copy)
+    goto fail;
   strcpy(name_copy, name);
   serializer->name=name_copy;
         
   label_copy=(char*)RAPTOR_CALLOC(cstring, strlen(label)+1, 1);
-  if(!label_copy) {
-    RAPTOR_FREE(raptor_serializer, serializer);
-    RAPTOR_FATAL1("Out of memory\n");
-  }
+  if(!label_copy)
+    goto fail;
   strcpy(label_copy, label);
   serializer->label=label_copy;
 
   if(mime_type) {
     mime_type_copy=(char*)RAPTOR_CALLOC(cstring, strlen(mime_type)+1, 1);
-    if(!mime_type_copy) {
-      RAPTOR_FREE(raptor_serializer, serializer);
-      RAPTOR_FATAL1("Out of memory\n");
-    }
+    if(!mime_type_copy)
+      goto fail;
     strcpy(mime_type_copy, mime_type);
     serializer->mime_type=mime_type_copy;
   }
 
   if(uri_string) {
     uri_string_copy=(unsigned char*)RAPTOR_CALLOC(cstring, strlen((const char*)uri_string)+1, 1);
-    if(!uri_string_copy) {
-    RAPTOR_FREE(raptor_serializer, serializer);
-    RAPTOR_FATAL1("Out of memory\n");
-    }
+    if(!uri_string_copy)
+      goto fail;
     strcpy((char*)uri_string_copy, (const char*)uri_string);
     serializer->uri_string=uri_string_copy;
   }
         
   if(alias) {
     alias_copy=(char*)RAPTOR_CALLOC(cstring, strlen(alias)+1, 1);
-    if(!alias_copy) {
-      RAPTOR_FREE(raptor_serializer, serializer);
-      RAPTOR_FATAL1("Out of memory\n");
-    }
+    if(!alias_copy)
+      goto fail;
     strcpy(alias_copy, alias);
     serializer->alias=alias_copy;
   }
 
+  if(raptor_sequence_push(serializers, serializer))
+    goto fail_noserializer; /* on error, serializer is already freed by the sequence */
+
   /* Call the serializer registration function on the new object */
   (*factory)(serializer);
   
 #if defined(RAPTOR_DEBUG) && RAPTOR_DEBUG > 1
   RAPTOR_DEBUG3("%s has context size %d\n", name, serializer->context_length);
 #endif
-  
-  raptor_sequence_push(serializers, serializer);
+
+  return;
+
+  /* Clean up on failure */
+  fail:
+  raptor_free_serializer_factory(serializer);
+  fail_noserializer:
+  raptor_finish();
+  RAPTOR_FATAL1("Out of memory\n");
 }
 
 
@@ -506,7 +513,7 @@ raptor_serialize_start_to_string(raptor_serializer *rdf_serializer,
  * Start serializing to a FILE*.
  * 
  * NOTE: This does not fclose the handle when it is finished.
-8
+ *
  * Return value: non-0 on failure.
  **/
 int
@@ -536,8 +543,8 @@ raptor_serialize_start_to_file_handle(raptor_serializer *rdf_serializer,
 /**
  * raptor_serialize_set_namespace:
  * @rdf_serializer: the #raptor_serializer
- * @uri: #raptor_uri of namespace
- * @prefix: prefix to use
+ * @uri: #raptor_uri of namespace or NULL
+ * @prefix: prefix to use or NULL
  *
  * set a namespace uri/prefix mapping for serializing.
  *
@@ -742,6 +749,10 @@ raptor_serializer_set_feature(raptor_serializer *serializer,
     case RAPTOR_FEATURE_NON_NFC_FATAL:
     case RAPTOR_FEATURE_WARN_OTHER_PARSETYPES:
     case RAPTOR_FEATURE_CHECK_RDF_ID:
+    case RAPTOR_FEATURE_HTML_TAG_SOUP:
+    case RAPTOR_FEATURE_MICROFORMATS:
+    case RAPTOR_FEATURE_HTML_LINK:
+    case RAPTOR_FEATURE_WWW_TIMEOUT:
 
     /* Shared */
     case RAPTOR_FEATURE_NO_NET:
@@ -835,6 +846,10 @@ raptor_serializer_set_feature_string(raptor_serializer *serializer,
     case RAPTOR_FEATURE_NON_NFC_FATAL:
     case RAPTOR_FEATURE_WARN_OTHER_PARSETYPES:
     case RAPTOR_FEATURE_CHECK_RDF_ID:
+    case RAPTOR_FEATURE_HTML_TAG_SOUP:
+    case RAPTOR_FEATURE_MICROFORMATS:
+    case RAPTOR_FEATURE_HTML_LINK:
+    case RAPTOR_FEATURE_WWW_TIMEOUT:
 
     /* Shared */
     case RAPTOR_FEATURE_NO_NET:
@@ -935,6 +950,10 @@ raptor_serializer_get_feature(raptor_serializer *serializer,
     case RAPTOR_FEATURE_NON_NFC_FATAL:
     case RAPTOR_FEATURE_WARN_OTHER_PARSETYPES:
     case RAPTOR_FEATURE_CHECK_RDF_ID:
+    case RAPTOR_FEATURE_HTML_TAG_SOUP:
+    case RAPTOR_FEATURE_MICROFORMATS:
+    case RAPTOR_FEATURE_HTML_LINK:
+    case RAPTOR_FEATURE_WWW_TIMEOUT:
 
     /* Shared */
     case RAPTOR_FEATURE_NO_NET:
@@ -1013,6 +1032,10 @@ raptor_serializer_get_feature_string(raptor_serializer *serializer,
     case RAPTOR_FEATURE_NON_NFC_FATAL:
     case RAPTOR_FEATURE_WARN_OTHER_PARSETYPES:
     case RAPTOR_FEATURE_CHECK_RDF_ID:
+    case RAPTOR_FEATURE_HTML_TAG_SOUP:
+    case RAPTOR_FEATURE_MICROFORMATS:
+    case RAPTOR_FEATURE_HTML_LINK:
+    case RAPTOR_FEATURE_WWW_TIMEOUT:
 
     /* Shared */
     case RAPTOR_FEATURE_NO_NET:
